Index: apps/app_directed_pickup.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_directed_pickup.c,v
retrieving revision 1.7
diff -u -r1.7 apps/app_directed_pickup.c
--- apps/app_directed_pickup.c	26 Oct 2005 19:48:14 -0000	1.7
+++ apps/app_directed_pickup.c	2 Nov 2005 06:42:29 -0000
@@ -54,7 +54,7 @@
 	int res = 0;
 	struct localuser *u = NULL;
 	struct ast_channel *origin = NULL, *target = NULL;
-	char *tmp = NULL, *exten = NULL, *context = NULL;
+	char *tmp = NULL, *context = NULL, *rest = data, *exten;
 	char workspace[256] = "";
 
 	if (ast_strlen_zero(data)) {
@@ -63,66 +63,69 @@
 	}
 
 	LOCAL_USER_ADD(u);
-	
-	/* Get the extension and context if present */
-	exten = data;
-	context = strchr(data, '@');
-	if (context) {
-		*context = '\0';
-		context++;
-	}
 
-	/* Find a channel to pickup */
-	origin = ast_get_channel_by_exten_locked(exten, context);
-	if (origin) {
-		ast_cdr_getvar(origin->cdr, "dstchannel", &tmp, workspace,
-			       sizeof(workspace), 0);
-		if (tmp) {
-			/* We have a possible channel... now we need to find it! */
-			target = ast_get_channel_by_name_locked(tmp);
+	while (!target && (exten = rest)) {
+		res = 0;
+		rest = strchr(exten, '&');
+		if (rest)
+			*rest++ = 0;
+
+		/* Get the extension and context if present */
+		context = strchr(exten, '@');
+		if (context)
+			*context++ = '\0';
+
+		/* Find a channel to pickup */
+		origin = ast_get_channel_by_exten_locked(exten, context);
+		if (origin) {
+			ast_cdr_getvar(origin->cdr, "dstchannel", &tmp, workspace,
+					sizeof(workspace), 0);
+			if (tmp) {
+				/* We have a possible channel... now we need to find it! */
+				target = ast_get_channel_by_name_locked(tmp);
+			} else {
+				ast_log(LOG_NOTICE, "No target channel found for %s.\n", exten);
+				res = -1;
+			}
+			ast_mutex_unlock(&origin->lock);
 		} else {
-			ast_log(LOG_DEBUG, "No target channel found.\n");
-			res = -1;
-		}
-		ast_mutex_unlock(&origin->lock);
-	} else {
-		ast_log(LOG_DEBUG, "No originating channel found.\n");
-	}
-	
-	if (res)
-		goto out;
-
-	if (target && (!target->pbx) && ((target->_state == AST_STATE_RINGING) || (target->_state == AST_STATE_RING))) {
-		ast_log(LOG_DEBUG, "Call pickup on chan '%s' by '%s'\n", target->name,
-			chan->name);
-		res = ast_answer(chan);
-		if (res) {
-			ast_log(LOG_WARNING, "Unable to answer '%s'\n", chan->name);
-			res = -1;
-			goto out;
+			ast_log(LOG_DEBUG, "No originating channel found.\n");
 		}
-		res = ast_queue_control(chan, AST_CONTROL_ANSWER);
-		if (res) {
-			ast_log(LOG_WARNING, "Unable to queue answer on '%s'\n",
-				chan->name);
-			res = -1;
-			goto out;
-		}
-		res = ast_channel_masquerade(target, chan);
-		if (res) {
-			ast_log(LOG_WARNING, "Unable to masquerade '%s' into '%s'\n", chan->name, target->name);
+
+		if (res)
+			continue;
+
+		if (target && (!target->pbx) && ((target->_state == AST_STATE_RINGING) || (target->_state == AST_STATE_RING))) {
+			ast_log(LOG_DEBUG, "Call pickup on chan '%s' by '%s'\n", target->name,
+					chan->name);
+			res = ast_answer(chan);
+			if (res) {
+				ast_log(LOG_WARNING, "Unable to answer '%s'\n", chan->name);
+				res = -1;
+				break;
+			}
+			res = ast_queue_control(chan, AST_CONTROL_ANSWER);
+			if (res) {
+				ast_log(LOG_WARNING, "Unable to queue answer on '%s'\n",
+						chan->name);
+				res = -1;
+				break;
+			}
+			res = ast_channel_masquerade(target, chan);
+			if (res) {
+				ast_log(LOG_WARNING, "Unable to masquerade '%s' into '%s'\n", chan->name, target->name);
+				res = -1;
+				break;
+			}
+		} else {
+			ast_log(LOG_NOTICE, "No call pickup possible for %s...\n", exten);
 			res = -1;
-			goto out;
 		}
-	} else {
-		ast_log(LOG_DEBUG, "No call pickup possible...\n");
-		res = -1;
 	}
-	/* Done */
- out:
+
 	if (target) 
 		ast_mutex_unlock(&target->lock);
-	
+
 	LOCAL_USER_REMOVE(u);
 
 	return res;
